# Generated by Django 3.2.12 on 2022-08-08 07:36

import django.contrib.postgres.indexes
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notification', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='message',
            name='mark',
            field=models.JSONField(blank=True, null=True, verbose_name='Message Mark'),
        ),
        migrations.AddIndex(
            model_name='message',
            index=django.contrib.postgres.indexes.GinIndex(fields=['mark'], name='mark_gin_index'),
        ),
    ]
